from __future__ import division
from display import DefaultDisplay
from screeninfo import get_monitors
from OpenGL.GL import *
from OpenGL.GLUT import *
from OpenGL.GLU import *

class DisplayOpenGL(DefaultDisplay):

    def __init__(self, matrix):
        DefaultDisplay.__init__(self, matrix)
        
        self.rectsize_x = 5
        self.rectsize_y = 1
        self.rectgap_x = 1
        self.rectgap_y = 2

        self.gl_matrix = [[[0, 0, 0] for j in range(self.matrix.height)] for i in range(self.matrix.width)]
        self.brightness_matrix = [[0 for j in range(self.matrix.height)] for i in range(self.matrix.width)]

        self.redisplayScheduled = False

        glutInit()
        #glutInitWindowSize(self.matrix.width * (self.rectsize_x + self.rectgap_x)  +100, self.matrix.height * (self.rectsize_y + self.rectgap_y)+100)
        monitor = get_monitors()[0]
        #glutInitWindowSize(monitor.width/2, monitor.height)
        #   glutInitWindowPosition(monitor.width/2, 0)

        glutInitDisplayMode(GLUT_BORDERLESS | GLUT_CAPTIONLESS)
        glutCreateWindow("LEDMatrix")
        glutDisplayFunc(self.render)
        glutInitDisplayMode(GLUT_BORDERLESS | GLUT_CAPTIONLESS)
        glClearColor(0.0,0.0,0.0,0.0)
        glColor3f(0.0,0.0, 0.0)
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()
        gluOrtho2D(0.0, self.matrix.width * (self.rectsize_x + self.rectgap_x), 0.0, self.matrix.height * (self.rectsize_y + self.rectgap_y))

    def run(self):
        pass

    def renderLoop(self):
        glutMainLoop()

    def _drawSquare(self, posX, posY, color):
        glColor3f(color[0], color[1], color[2])
        glRecti(posX * self.rectsize_x + (posX-1)*self.rectgap_x, posY * self.rectsize_y + (posY-1)*self.rectgap_y,
                (posX+1) * self.rectsize_x + (posX-1)*self.rectgap_x, (posY+1) * self.rectsize_y + (posY-1)*self.rectgap_y)
    def _drawOverlay(self, posX, posY, brightness):
        glColor4f(0, 0, 0, brightness)
        glRecti(posX * self.rectsize_x + (posX-1)*self.rectgap_x, posY * self.rectsize_y + (posY-1)*self.rectgap_y,
                (posX+1) * self.rectsize_x + (posX-1)*self.rectgap_x, (posY+1) * self.rectsize_y + (posY-1)*self.rectgap_y)

    # Callback from LEDMatrix
    def onLEDChange(self, index):
        led = self.matrix.getLEDByIndex(index)
        heightPos = index % self.matrix.longestDimension
        widthPos = index // self.matrix.longestDimension
        self.gl_matrix[widthPos][heightPos] = led.colour
        self.brightness_matrix[widthPos][heightPos] = led.brightness
        if not self.redisplayScheduled:
            glutPostRedisplay()
            self.redisplayScheduled = True

    def render(self):
        self.redisplayScheduled = False
        glClear(GL_COLOR_BUFFER_BIT)
        glEnable(GL_BLEND)
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
        for i in range(len(self.gl_matrix)):
            for j in range(len(self.gl_matrix[i])):
                self._drawSquare(i, j+1, self.gl_matrix[i][j])
                self._drawOverlay(i, j+1, 1 - self.brightness_matrix[i][j]/4)
                
        glFlush()
        glutPostRedisplay()
