import colorsys
""" MATRICE PHYSIQUE DES LEDS. 1 liste = 1 bloc avec ID de ses LEDs:

[0,1,2,3]	               [6,7,8,9,10]	        [13,14,15,16]	    [19,20,21,22]	      [26,27,28,29]	    	[32,33,34,35]	  	[38,39,40,41]  		[44,45,46,47]
[91,92,93,94,95]	       [85,86,87,88]	    [79,80,81,82]	    [73,74,75,76]	      [66,67,68,69]	    	[60,61,62,63]	  	[54,55,56,57]	    [48,49,50,51]
[96,97,98,99,100]	       [103,104,105,106]	[109,110,111,112]	[115,116,117,118]	  [122,123,124,125]		[128,129,130,131] 	[134,135,136,137]	[141,140,141,142,143]
[187,188,189,190,191]	   [181,182,183,184]	[175,176,177,178]	[169,170,171,172]	  [162,163,164,165]		[156,157,158,159] 	[150,151,152,153]	[144,145,146,147]
[193,194,195,196]	       [199,200,201,202]	[205,206,207,208]	[211,212,213,214,215] [218,219,220,221]		[224,225,226,227] 	[230,231,232,233]	[236,237,238,239,240,239]
[283,284,285,286,287]	   [277,278,279,280]	[271,272,273,274]	[265,266,267,268]	  [258,259,260,261]		[252,253,254,255] 	[246,247,248,249]	[240,241,242]
[289,290,291,292]	       [295,296,297,298]	[301,302,303,304]	[307,308,309,310,311] [314,315,316,317]		[321,322,323]	  	[327,328,329]	    [333,334,335]
[379,380,381,382]	       [373,374,375,376]	[367,368,369,370]	[361,362,363,364]	  [354,355,356,357]		[348,349,350,351] 	[342,343,344]	    [336,337,338]
[383,384,385,386,387,388]  [391,392,393,394]	[397,398,399,400]	[403,404,405,406]	  [410,411,412,413]		[416,417,418,419] 	[423,424,425]	    [429,430,431]
[475,476,477,478]	       [469,470,471,472]	[463,464,465,466]	[457,458,459,460]	  [451,452,453,454]		[444,445,446,447] 	[438,439,440,441]	[432,433,434]

"""


class LED:

	def __init__(self, index):
		self.colour = [0, 0, 0]
		self.index = index
		self.preparing = False
		self.brightness = 4
		self.listeners = []

	def setColourAuto(self, colour_mode, a, b, c, d=0):
		if colour_mode == "hsv":
			self.setColourHSV(a, b, c)
		elif colour_mode == "hls":
			self.setColourHLS(a, b, c)
		elif colour_mode == "rvb":
			self.setColourRVB(a, b, c)
		else:
			print("Unknown colour scheme " + str(colour_mode))

	def setBrightness(self, b):
		self.brightness = b

        def setColourRVBIndex(self, index, value):
                self.preparint = False
                self.colour[index] = value
                self.notifyListeners()

	def setColourRVB(self, r, v, b):
		self.colour = [r, v, b]
		self.notifyListeners()

	def setColourHSV(self, h, s, v):
		self.colour = list(colorsys.hsv_to_rgb(h, s, v))
		self.notifyListeners()

	def setColourHLS(self, h, l, s):
		self.colour = list(colorsys.hls_to_rgb(h, l, s))
		self.notifyListeners()

	def getColourHSV(self):
		return colorsys.rgb_to_hsv(*self.colour)

	def notifyListeners(self):
		if not self.preparing:
			for listener in self.listeners:
				listener.onLEDChange(self.index)

	def send(self):
		self.preparing = False
		self.notifyListeners()

	def setPreparing(self, flag):
		self.preparing = flag

	def isPreparing(self):
		return self.preparing

	def registerListener(self, listener):
		self.listeners.append(listener)

	def removeListener(self, listener):
		self.listeners.remove(listener)

class LEDMatrixIterator:

	def __init__(self, ledMatrix):
		self.ledMatrix = ledMatrix
		self.posX = 0
		self.posY = 0
		self.first = True

	def __iter__(self):
		return self
	def next(self):
		return self.__next__()
	def __next__(self):
		if self.first:
			self.first = False
			return (0 , 0, self.ledMatrix.getLED(0, 0))

		self.posY += 1
		if (self.posY >= self.ledMatrix.height):
			self.posX += 1
			self.posY = 0
			if (self.posX >= self.ledMatrix.width):
				raise StopIteration
		return (self.posX, self.posY, self.ledMatrix.getLED(self.posX, self.posY))
			

class LEDMatrix:

	def __init__(self, width, height):
		self.width = width
		self.height = height
		self.longestDimension = self.width if self.width >= self.height else self.height
		self.LEDs = [[LED(self.longestDimension*i+j) for j in range(self.height)] for i in range(self.width)]
		self.brightness = 255
		
		self.listeners = []

		# Register colour change callbacks
		#for column in self.LEDs:
		#	for led in column:
		#		led.registerListener(self)

        def reset(self):
            for column in self.LEDs:
                for led in column:
                    led.setColourRVB(0,0,0)

        def register(self, listener):
		for column in self.LEDs:
			for led in column:
				led.registerListener(listener)
	def iterator(self):
		return LEDMatrixIterator(self)

	def getLEDByIndex(self, ledIndex):
		heightPos = ledIndex % self.longestDimension
		widthPos = ledIndex // self.longestDimension
		return self.getLED(widthPos, heightPos)

	def getLED(self, column, height):
		assert column >= 0 and column < self.width
		assert height >= 0 and height < self.height

		return self.LEDs[column][height]

	def setBrightness(self, brightness):
		self.brightness = brightness
                for listener in self.listeners:
                    listener.onBrightnessUpdate(brightness)

	def onLEDChange(self, ledIndex):
		self.notifyListeners()

	def notifyListeners(self):
		for listener in self.listeners:
			listener.onMatrixUpdate(self.LEDs)

	def registerListener(self, listener):
		self.listeners.append(listener)

	def removeListener(self, listener):
		self.listeners.remove(listener)
