# -*- coding: latin-1 -*-
from ledmatrix import LEDMatrix, LED
from audio import AudioHandler, AudioStreamPyAudio
from processing.processing import Processing
from display.display_opengl import DisplayOpenGL
from display.display_neopixel import DisplayNeoPixel
from communication.comm import ServerSocket
import sys
import os
import signal
from dmx import DMX

class Main:

    def __init__(self):
        # Toi, tu dégages
        os.system("killall lxpanel")
        self.dmx_control = False
        # Source audio
        self.audio = AudioStreamPyAudio(device=0)

        # Paramètres de la matrice de LEDs
        self.led_matrix = LEDMatrix(8, 10)

        # Calculs liés à la FFT (dans un thread à part)
        self.audio_handler = AudioHandler(self.led_matrix.width, self.led_matrix.height, self.audio)
        self.audio_handler.start()

        # Tout le processing après analyse par FFT
        self.processing = Processing(self.led_matrix, 60)
        self.audio_handler.registerListener(self.processing)
        self.processing.start()

        # Affichage (OpenGL ici, leds à terme)
        #gui.init()
        #self.display = DisplayOpenGL(self.led_matrix)
        self.display2 = DisplayNeoPixel(self.led_matrix, RATE=60)
        #self.led_matrix.register(self.display)
        self.led_matrix.register(self.display2)
        self.com = ServerSocket(self)
        self.com.start()
        #self.display.start()
        self.display2.start()
        #self.display.renderLoop()
        
        self.dmx = DMX(self.led_matrix, self)
        self.dmx.start()

    def setDMXControl(self, status):
        print("Setting DMX control status to ", status)
        if status == False and self.dmx_control:
            self.led_matrix.reset()
            self.audio_handler.registerListener(self.processing)
            self.processing.resume()
            self.audio_handler.resume()
            self.dmx_control = False
        elif status == True and not self.dmx_control:
            self.led_matrix.reset()
            self.audio_handler.removeListener(self.processing)
            self.processing.pause()
            self.audio_handler.pause()
            self.dmx_control = True


if __name__ == '__main__':
    def sigint(sig, frame):
        print("Received Keyboard Interrupt signal, shutting down in a clean way")
        main.processing.stop()
        main.com.stop()
        sys.exit(0)
    main = Main()
    signal.signal(signal.SIGINT, sigint)
