from processing import DefaultHandler

import time

class TextHandler(DefaultHandler):
    
    ALPHABET_6x6 = {
        " ":    [[0 for _ in range(6)] for _ in range(6)],
        "A":    [   [0, 1, 1, 1, 1, 0],
                    [1, 1, 0, 0, 1, 1],
                    [1, 1, 0, 0, 1, 1],
                    [1, 1, 1, 1, 1, 1],
                    [1, 1, 0, 0, 1, 1],
                    [1, 1, 0, 0, 1, 1]
                ],
        "B":    [   [1, 1, 1, 1, 0, 0],
                    [1, 1, 0, 1, 1, 0],
                    [1, 1, 1, 1, 1, 1],
                    [1, 1, 0, 0, 1, 1],
                    [1, 1, 0, 0, 1, 1],
                    [1, 1, 1, 1, 0, 0]
                ],
        "C":    [   [0, 1, 1, 1, 1, 0],
                    [1, 1, 0, 0, 1, 1],
                    [1, 1, 0, 0, 1, 1],
                    [1, 1, 0, 0, 0, 0],
                    [1, 1, 0, 0, 1, 1],
                    [0, 1, 1, 1, 1, 0]
                ],
        "D":    [   [1, 1, 1, 1, 1, 0],
                    [1, 1, 0, 0, 1, 1],
                    [1, 1, 0, 0, 1, 1],
                    [1, 1, 0, 0, 1, 1],
                    [1, 1, 0, 0, 1, 1],
                    [1, 1, 1, 1, 1, 0]
                ],
        "E":    [   [1, 1, 1, 1, 1, 1],
                    [1, 1, 0, 0, 1, 1],
                    [1, 1, 1, 0, 0, 0],
                    [1, 1, 0, 0, 0, 0],
                    [1, 1, 0, 0, 1, 1],
                    [1, 1, 1, 1, 1, 1]
                ],
        "F":    [   [1, 1, 1, 1, 1, 1],
                    [1, 1, 0, 0, 1, 1],
                    [1, 1, 1, 0, 0, 0],
                    [1, 1, 0, 0, 0, 0],
                    [1, 1, 0, 0, 0, 0],
                    [1, 1, 0, 0, 0, 0]
                ],
        "G":    [   [0, 1, 1, 1, 1, 0],
                    [1, 1, 0, 0, 1, 1],
                    [1, 1, 0, 0, 0, 0],
                    [1, 1, 0, 1, 1, 1],
                    [1, 1, 0, 0, 1, 1],
                    [1, 1, 1, 1, 1, 1]
                ],
        "H":    [   [1, 1, 0, 0, 1, 1],
                    [1, 1, 0, 0, 1, 1],
                    [1, 1, 0, 0, 1, 1],
                    [1, 1, 1, 1, 1, 1],
                    [1, 1, 0, 0, 1, 1],
                    [1, 1, 0, 0, 1, 1]
                ],
        "I":    [   [1, 1, 1, 1, 1, 1],
                    [0, 0, 1, 1, 0, 0],
                    [0, 0, 1, 1, 0, 0],
                    [0, 0, 1, 1, 0, 0],
                    [0, 0, 1, 1, 0, 0],
                    [1, 1, 1, 1, 1, 1]
                ],
        "J":    [   [1, 1, 1, 1, 1, 1],
                    [0, 0, 1, 1, 0, 0],
                    [0, 0, 1, 1, 0, 0],
                    [1, 0, 1, 1, 0, 0],
                    [1, 1, 1, 1, 0, 0],
                    [1, 1, 1, 1, 0, 0]
                ],
        "K":    [   [1, 1, 0, 0, 1, 0],
                    [1, 1, 0, 1, 0, 0],
                    [1, 1, 1, 0, 0, 0],
                    [1, 1, 1, 1, 0, 0],
                    [1, 1, 0, 1, 1, 0],
                    [1, 1, 0, 0, 1, 1]
                ],
        "L":    [   [1, 1, 0, 0, 0, 0],
                    [1, 1, 0, 0, 0, 0],
                    [1, 1, 0, 0, 0, 0],
                    [1, 1, 0, 0, 0, 0],
                    [1, 1, 0, 0, 1, 1],
                    [1, 1, 1, 1, 1, 1]
                ],
        "M":    [   [1, 1, 0, 0, 1, 1],
                    [1, 1, 1, 1, 1, 1],
                    [1, 1, 1, 1, 1, 1],
                    [1, 0, 1, 0, 1, 1],
                    [1, 0, 0, 0, 1, 1],
                    [1, 0, 0, 0, 1, 1]
                ],
        "N":    [   [1, 1, 0, 0, 1, 1],
                    [1, 1, 1, 0, 1, 1],
                    [1, 1, 1, 0, 1, 1],
                    [1, 1, 0, 1, 1, 1],
                    [1, 1, 0, 1, 1, 1],
                    [1, 1, 0, 0, 1, 1]
                ],
        "O":    [   [0, 1, 1, 1, 1, 0],
                    [1, 1, 1, 0, 1, 1],
                    [1, 1, 0, 0, 1, 1],
                    [1, 1, 0, 0, 1, 1],
                    [1, 1, 0, 1, 1, 1],
                    [0, 1, 1, 1, 1, 0]
                ],
        "P":    [   [1, 1, 1, 1, 1, 0],
                    [1, 1, 0, 0, 1, 1],
                    [1, 1, 0, 0, 1, 1],
                    [1, 1, 1, 1, 1, 0],
                    [1, 1, 0, 0, 0, 0],
                    [1, 1, 0, 0, 0, 0]
                ],
        "Q":    [   [0, 1, 1, 1, 1, 0],
                    [1, 1, 0, 0, 1, 1],
                    [1, 1, 0, 0, 1, 1],
                    [1, 1, 0, 0, 1, 1],
                    [1, 1, 0, 1, 1, 1],
                    [0, 1, 1, 1, 1, 1]
                ],
        "R":    [   [1, 1, 1, 1, 1, 1],
                    [1, 1, 0, 0, 1, 1],
                    [1, 1, 0, 0, 1, 1],
                    [1, 1, 1, 1, 1, 0],
                    [1, 1, 0, 0, 1, 1],
                    [1, 1, 0, 0, 1, 1]
                ],
        "S":    [   [0, 1, 1, 1, 1, 1],
                    [1, 1, 0, 0, 1, 1],
                    [1, 1, 1, 0, 0, 0],
                    [0, 0, 0, 1, 1, 1],
                    [1, 1, 0, 0, 1, 1],
                    [1, 1, 1, 1, 1, 0]
                ],
        "T":    [   [1, 1, 1, 1, 1, 1],
                    [0, 0, 1, 1, 0, 0],
                    [0, 0, 1, 1, 0, 0],
                    [0, 0, 1, 1, 0, 0],
                    [0, 0, 1, 1, 0, 0],
                    [0, 0, 1, 1, 0, 0]
                ],
        "U":    [   [1, 1, 0, 0, 1, 1],
                    [1, 1, 0, 0, 1, 1],
                    [1, 1, 0, 0, 1, 1],
                    [1, 1, 0, 0, 1, 1],
                    [1, 1, 0, 0, 1, 1],
                    [0, 1, 1, 1, 1, 0]
                ],
        "V":    [   [1, 1, 0, 0, 1, 1],
                    [1, 1, 0, 0, 1, 1],
                    [0, 1, 1, 0, 1, 0],
                    [0, 1, 1, 0, 1, 0],
                    [0, 0, 1, 1, 0, 0],
                    [0, 0, 1, 1, 0, 0]
                ],
        "W":    [   [1, 0, 0, 0, 1, 1],
                    [1, 0, 0, 0, 1, 1],
                    [1, 0, 0, 0, 1, 1],
                    [1, 0, 1, 0, 1, 1],
                    [1, 0, 1, 0, 1, 1],
                    [0, 1, 0, 1, 1, 0]
                ],
        "X":    [   [1, 1, 0, 0, 1, 1],
                    [1, 1, 1, 0, 1, 1],
                    [0, 0, 1, 1, 0, 0],
                    [0, 0, 1, 1, 0, 0],
                    [1, 1, 0, 1, 1, 1],
                    [1, 1, 0, 0, 1, 1]
                ],
        "Y":    [   [1, 1, 0, 0, 1, 1],
                    [1, 1, 0, 0, 1, 1],
                    [0, 1, 1, 1, 1, 0],
                    [0, 0, 1, 1, 0, 0],
                    [0, 0, 1, 1, 0, 0],
                    [0, 0, 1, 1, 0, 0]
                ],
        "Z":    [   [1, 1, 1, 1, 1, 1],
                    [1, 1, 0, 0, 1, 1],
                    [0, 0, 0, 1, 0, 0],
                    [0, 0, 1, 0, 0, 0],
                    [1, 1, 0, 0, 1, 1],
                    [1, 1, 1, 1, 1, 1]
                ],
        "!":    [   [0, 0, 1, 1, 0, 0],
                    [0, 0, 1, 1, 0, 0],
                    [0, 0, 1, 1, 0, 0],
                    [0, 0, 1, 1, 0, 0],
                    [0, 0, 0, 0, 0, 0],
                    [0, 0, 1, 1, 0, 0]
                ],
        "0":    [   [0, 1, 1, 1, 1, 0],
                    [1, 1, 0, 0, 1, 1],
                    [1, 1, 0, 1, 1, 1],
                    [1, 1, 1, 0, 1, 1],
                    [1, 1, 0, 0, 1, 1],
                    [0, 1, 1, 1, 1, 0]
                ],
        "1":    [   [0, 0, 1, 1, 0, 0],
                    [1, 1, 1, 1, 0, 0],
                    [0, 0, 1, 1, 0, 0],
                    [0, 0, 1, 1, 0, 0],
                    [0, 0, 1, 1, 0, 0],
                    [1, 1, 1, 1, 1, 1]
                ],
        "2":    [   [0, 1, 1, 1, 1, 0],
                    [1, 1, 0, 0, 1, 1],
                    [0, 0, 0, 0, 1, 1],
                    [0, 0, 1, 1, 0, 0],
                    [1, 1, 0, 0, 0, 0],
                    [1, 1, 1, 1, 1, 1]
                ],
        "3":    [   [1, 1, 1, 1, 1, 1],
                    [0, 0, 0, 0, 1, 0],
                    [0, 0, 1, 1, 0, 0],
                    [0, 0, 0, 1, 1, 1],
                    [1, 1, 0, 0, 1, 1],
                    [0, 1, 1, 1, 1, 0]
                ],
        "4":    [   [1, 1, 0, 1, 1, 0],
                    [1, 1, 0, 1, 1, 0],
                    [1, 1, 1, 1, 1, 1],
                    [0, 0, 0, 1, 1, 0],
                    [0, 0, 0, 1, 1, 0],
                    [0, 0, 0, 1, 1, 0]
                ],
        "5":    [   [1, 1, 1, 1, 1, 1],
                    [1, 1, 0, 0, 0, 0],
                    [1, 1, 1, 1, 1, 0],
                    [0, 0, 0, 0, 1, 1],
                    [1, 1, 0, 0, 1, 1],
                    [0, 1, 1, 1, 1, 0]
                ],
        "6":    [   [0, 1, 1, 1, 1, 0],
                    [1, 1, 0, 0, 0, 0],
                    [1, 1, 1, 1, 1, 0],
                    [1, 1, 0, 0, 1, 1],
                    [1, 1, 0, 0, 1, 1],
                    [0, 1, 1, 1, 1, 0]
                ],
        "7":    [   [1, 1, 1, 1, 1, 1],
                    [1, 1, 0, 0, 1, 1],
                    [0, 0, 0, 1, 1, 0],
                    [0, 0, 0, 1, 1, 0],
                    [0, 0, 1, 1, 0, 0],
                    [0, 0, 1, 1, 0, 0]
                ],
        "8":    [   [0, 0, 1, 1, 1, 0],
                    [0, 1, 1, 0, 1, 1],
                    [0, 1, 1, 1, 1, 1],
                    [1, 1, 0, 0, 1, 1],
                    [1, 1, 0, 0, 1, 1],
                    [0, 1, 1, 1, 1, 0]
                ],
        "9":    [   [0, 1, 1, 1, 1, 0],
                    [1, 1, 0, 0, 1, 1],
                    [1, 1, 1, 1, 1, 1],
                    [0, 0, 0, 1, 1, 0],
                    [0, 0, 1, 1, 0, 0],
                    [0, 0, 1, 1, 0, 0]
                ],
    }
    def __init__(self, width, height):
        DefaultHandler.__init__(self, width, height)

        self.text = " "
        self.textPos = 0
        self.scrollRate = 1
        self.lastScroll = time.time()
        self.colour = [0.5, 0.8]

        self.pauseTime=0.15

        self.values = []

        self.empty = []

        for i in range(6):
            for j in range(6):
                self.empty.append((1 + i, 5 - j + 2, [0, 0, 0]))

    def setColour(self, h, s):
        self.colour = [h, s]

    def setText(self, text):
        self.text = text.upper()
        self.textPos = 0

    def setScrollRate(self, rate):
        self.scrollRate = rate

    def getValues(self):
        if time.time() - self.lastScroll <= 0.1:
            return (self.empty, "hsv")

        if time.time() - self.lastScroll >= 1/self.scrollRate:
            self.lastScroll = time.time()
            if (self.textPos >= len(self.text)):
                self.textPos = 0

            self.values = []

            pixels = self.ALPHABET_6x6[self.text[self.textPos]]
            for i in range(len(pixels)):
                for j in range(len(pixels[i])):
                    self.values.append((1+i, len(pixels[0]) - 1 - j + 2, [0, 1, pixels[j][i]*0.8], 4))
            self.textPos += 1
        return (self.values, "hsv")
